#!/bin/bash
# This script requires readline, sed, wget, curl, gpg, fakeroot, dpkg, rsync, 
# finger, and the mkisofs packages on the runtime environment to function 
# correctly and is designed to run in a Linux environment. Creation of a VM 
# also requires installation of qemu and vncviewer
#
# NOTE: Install readline-common and libreadline6 on Ubunut for readline
#
###############################################################################
#
# Copyright 2010, PSInd, LLC, Framingham, MA 01702 (http://psind.com)
#
#
# Except as explicitly allowed under each specific copyright or license, this 
# materials may not be reproduced in whole or in part, in any form or by any 
# means, including photocopy, electronic storage and retrieval, or translation 
# into any other language without the express written consent of the copyright 
# holder. This script may be freely redistributed provided this copyright 
# notice is maintained within the content of the script and any revisions are 
# clearly annotated as such with an author and date of change.
#
# This script is provided "as is" without warranty of any kind, express or 
# implied, including, but not limited to, the implied warranties of fitness 
# for a particular purpose, and non-infringement.
# 
# In no event shall PSInd, LLC. be liable to you or any third party for any 
# direct or indirect, special, incidental, or consequential damages in 
# connection with or arising from errors, omissions, delays or other cause of 
# action that may be attributed to your use of any product, support, services, 
# information or software from this site, including, but not limited to, lost 
# profits or lost data, even if PSInd, LLC had been advised of the possibility 
# of such damages.
#
###############################################################################

#
# Modifiable script variables
#

# Password for the 'prpcadmin' user automatically created on the installed 
# Ubuntu system
prpcadmin_os_password=changeme
ldap_adm_passwd=${prpcadmin_os_passwd}

# PRPC distribution directory (mediadir) and version (no '-' characters 
#   allowed)
prpc_mediadir='/data/Pegasystems/PRPC/PRPC_6.1sp1'
prpc_version='6.1sp1'

# The schema name for the database with credentials, same credentials are
#   used for tomcat6
prpc_dbtsname=prpc61sp1
prpc_dbname=XE
prpc_db_userid=prpc61sp1
prpc_db_passwd=changeme
ora_system_passwd=${prpc_db_passwd}
ora_appexp_port=8191
sys_domain=`domainname`
sys_hostname=prpc61sp1svr

# Identify a target directory to contain all working files - will be created
prpcubudir=~/prpcbuntu

# Distribution packager info - default to current user
dist_packaging_user="`finger -l $LOGNAME | grep Name: | cut -f 3 -d ':' | cut -c2-`"
if [ "Z${dist_packaging_user}" = "Z" ] ; then dist_packaging_user=`logname`; fi
dist_packaging_email=`logname`@${sys_domain}

# ubuntu_iso - Empty string indicates download and discard, absolute path
#   indicates desire to use a local ISO or save after download
ubuntu_iso='/data/Linux/Ubuntu/ubuntu-10.04-server-i386.iso'
prpcbuntu_iso=${prpcubudir}/prpcbuntu.iso

# Locally accessible Oracle 10g XE debian package - must have a value. If the
#   file does not exist, it will be saved to this location after download.
oracle_deb='/data/Linux/oracle/10g/x86/oracle-xe_10.2.0.1-1.1_i386.deb'

# Identify the core distribution (server, desktop, netbook) & bits (32, 64)
ubuntu_dist=server
ubuntu_bits=32

# Identify the language and locale for unattended install
ubuntu_lang=en
ubuntu_locale=US
ubuntu_conlayout=us

##############################################################################
#################### DO NOT CHANGE ANYTHING BELOW THIS LINE ##################
##############################################################################
#
# Begin image creation script 
#

ps_end=`wc -l "$0" | cut -f 1 -d ' '`
ps_start=`grep -n \#\#\#\ START\ PRESEED\ INCLUDE\ \#\#\# "$0" | cut -f 1 -d ':'`
pi_start=`grep -n \#\#\#\ START\ PRPC\ INSTALL\ INCLUDE\ \#\#\# "$0" | cut -f 1 -d ':'`
pr_start=`grep -n \#\#\#\ START\ PRPC\ REMOVE\ INCLUDE\ \#\#\# "$0" | cut -f 1 -d ':'`
pbl_start=`grep -n \#\#\#\ START\ PRPC\ BE\ LDIF\ INCLUDE\ \#\#\# "$0" | cut -f 1 -d ':'`
pl_start=`grep -n \#\#\#\ START\ PRPC\ LDIF\ INCLUDE\ \#\#\# "$0" | cut -f 1 -d ':'`

vm_format=qcow2
keep_prpc_wd='yes'
prpcbuntu_iso_skip=0
prpcbuntu_confirm_settings=0

getAndExtractUbunutu()
{
    pushd "${prpcubudir}"

    # Get the source ubunutu image and extract to working directory
    if [ "Z" = "Z${ubuntu_iso}" ] ; then 
        # No permanent ISO location identified - download and discard
	wget --post-data 'distro=${ubuntu_dist}&bits=${ubuntu_bits}' http://www.ubuntu.com/start-download
	ubuntu_iso=`ls -1 *.iso`
    elif [ -f "${ubuntu_iso}" ] ; then
        # Ubuntu ISO exists locally - use local ISO
	ln -s ${ubuntu_iso} .
    else
    # download and save to identified local ISO location
	wget --post-data 'distro=${ubuntu_dist}&bits=${ubuntu_bits}' http://www.ubuntu.com/start-download
	mv *.iso ${ubuntu_iso}
	ln -s ${ubuntu_iso} .
    fi

    echo "Creating ISO working directory..."
    dist_image=`basename ${ubuntu_iso}`
    dist_image_mnt=`mktemp -d -p ${prpcubudir}`
    sudo mount -o loop ${dist_image} ${dist_image_mnt}
    mkdir -p cd-image
    sudo rsync --delete -av ${dist_image_mnt}/ cd-image
    
    # Unmount the downloaded ISO - delete only if it was linked
    #sudo umount ${dist_image_mnt} 2> /dev/null && rmdir ${dist_image_mnt}
    sudo umount ${dist_image_mnt} && rmdir ${dist_image_mnt}

    if [ -h ${dist_image} ] ; then rm ${dist_image}; fi

    popd
}

repackageSigningKeys()
{
    if [ ! -d ${prpcubudir}/gpgdir ] ; then
	ubukring=`ls ${prpcubudir}/cd-image/ubuntu/pool/main/u/ubuntu-keyring/ubuntu-keyring*.deb | xargs -ifoo basename foo | cut -c1-25`
	sudo mkdir ${prpcubudir}/gpgdir && sudo chmod 777 ${prpcubudir}/gpgdir
	pushd ${prpcubudir}/gpgdir
	
	printf "\n\nIt is necessary to sign the modified distribution using GnuPG\n\
  We need the name and email of the person responsible for the package \n\
  generation to create the package using the GnuPG key you are about to \n\
  generate:\n\n"
	while [ "Z${dist_packaging_user}" = "Z" ] ; do
	    read -p "Enter the full name of the packager: " dist_packaging_user
	done
	while [ "Z${dist_packaging_email}" = "Z" ] ; do
	    read -p "Enter the email address of the packager: " dist_packaging_email
	done
	
        # Assume the first key with a secret key available is the signing key
	MYSIGNINGKEY=`gpg --list-secret-keys | grep ^uid | cut -c16- | xargs echo | cut -f 1 -d '<' | cut -f 1 -d '('`
	
	if [ "Z${MYSIGNINGKEY}" = "Z" ] ; then
	    printf "\n\nA GPG signing key could not be located for your current account.\n\
  We are now generating a new GnuPG key for signing using the gpg tool - \n\
  This will create a permanent signing key under the logged in account.\n\
  Follow the instruction, and accept the defaults for most questions. \n\
  Keep track of the secret / password, you will need it to sign packages \n\
  in the steps following and it will be available on your user's keyring \n\
  for any future code signing operations with gpg. You should enter the \n\
  same name and email you used above for the user ID / Real Name and the \n\
  Email Address:\n\n"
	    gpg --gen-key
	    
            # Assume the first key with a secret key available is the signing key
	    MYSIGNINGKEY=`gpg --list-secret-keys | grep ^uid | cut -c16- | xargs echo | cut -f 1 -d '<' | cut -f 1 -d '('`
	    printf "Created new GPG signing key.\n"
	else
	    printf "Existing GPG key found.\n"
	fi
	
	printf "Regenerating Ubuntu key ring with new GPG key: %s\n\n" \
	    "${MYSIGNINGKEY}"
	EXTRASKEYID=`gpg --list-keys ${MYSIGNINGKEY} | grep ^pub | cut -f 2 -d '/' | cut -f 1 -d ' '`
	
	wget http://archive.ubuntu.com/ubuntu/pool/main/u/ubuntu-keyring/${ubukring}.tar.gz 
	tar -xzvf ${ubukring}.tar.gz && rm -f ${ubukring}.tar.gz
	pushd ubuntu-keyring-*/keyrings
	gpg --import < ubuntu-archive-keyring.gpg
	popd
	popd
	pushd ${prpcubudir}/gpgdir/ubuntu-keyring-*
    else
	pushd ${prpcubudir}/gpgdir/ubuntu-keyring-*
	gpg --import < keyrings/ubuntu-archive-keyring.gpg
    fi
    
    gpg --export FBB75451 437D05B5 ${EXTRASKEYID} > keyrings/ubuntu-archive-keyring.gpg
    sudo dpkg-buildpackage -ai386 -rfakeroot -m"${dist_packaging_user} <${dist_packaging_email}>" -k${EXTRASKEYID}
    sudo cp -v ../ubuntu-keyring*deb ${prpcubudir}/cd-image/pool/main/u/ubuntu-keyring
    popd
}

createFTPArchive()
{
    printf 'Dir {\n\
  ArchiveDir "${image_dir}/";\n\
};\n\
\n\
TreeDefault {\n\
  Directory "pool/";\n\
};\n\
\n\
BinDirectory "pool/main" {\n\
  Packages "dists/${distname}/main/binary-i386/Packages.gz";\n\
  BinOverride "${indices_dir}/override.${distname}.main";\n\
  ExtraOverride "${indices_dir}/override.${distname}.extra.main";\n\
};\n\
\n\
BinDirectory "pool/restricted" {\n\
 Packages "dists/${distname}/restricted/binary-i386/Packages.gz";\n\
 BinOverride "${indices_dir}/override.${distname}.restricted";\n\
};\n\
\n\
Default {\n\
  Packages {\n\
    Extensions ".deb";\n\
    Compress ". gzip";\n\
  };\n\
};\n\
\n\
Contents {\n\
  Compress "gzip";\n\
};' > ${archive_dir}/apt-ftparchive-deb.conf

    printf 'Dir {\n\
  ArchiveDir "${image_dir}/";\n\
};\n\
\n\
TreeDefault {\n\
  Directory "pool/";\n\
};\n\
\n\
BinDirectory "pool/extras" {\n\
  Packages "dists/${distname}/extras/binary-i386/Packages";\n\
};\n\
\n\
Default {\n\
  Packages {\n\
    Extensions ".deb";\n\
    Compress ". gzip";\n\
  };\n\
};\n\
\n\
Contents {\n\
  Compress "gzip";\n\
};' > ${archive_dir}/apt-ftparchive-extras.conf

    printf 'APT::FTPArchive::Release::Origin "Ubuntu";
APT::FTPArchive::Release::Label "Ubuntu";
APT::FTPArchive::Release::Suite "${distname}";
APT::FTPArchive::Release::Version "${distversion}";
APT::FTPArchive::Release::Codename "${distname}";
APT::FTPArchive::Release::Architectures "i386";
APT::FTPArchive::Release::Components "main restricted extras";
APT::FTPArchive::Release::Description "Ubuntu ${distversion} PRPC";' > ${archive_dir}/release.conf

    pushd "${image_dir}"
    APT_CONF="${archive_dir}/release.conf"
#    apt-ftparchive -c $APTCONF generate "${archive_dir}/apt-ftparchive-deb.conf"
#    apt-ftparchive -c $APTCONF generate "${archive_dir}/apt-ftparchive-extras.conf"
#    sudo apt-ftparchive -c $APTCONF release "${image_dir}/dists/${distname}" > "${image_dir}/dists/${distname}/Release"
    
    sudo chmod 666 "dists/${distname}/Release.gpg" md5sum.txt
    gpg --default-key "${MYSIGNINGKEYID}" --output "dists/${distname}/Release.gpg" -ba "dists/${distname}/Release"
    sudo find . -type f -print0 | sudo xargs -0 md5sum > md5sum.txt
    popd

    rm -rf "${indices_dir}" "${archive_dir}"
}

repackageUbuntuPackages()
{
    indices_dir="${prpcubudir}/indices"
    archive_dir="${prpcubudir}/archive"
    image_dir="${prpcubudir}/cd-image"
    mkdir -p "${indices_dir}" "${archive_dir}"

    # Get the default indices configs from the Ubunutu site...
    pushd "${indices_dir}"
    for SUFFIX in extra.main main main.debian-installer restricted restricted.debian-installer; do
	wget http://archive.ubuntu.com/ubuntu/indices/override.${distname}.$SUFFIX
    done
    popd

    pushd "${image_dir}/dists/${distname}"
    # Create the extras packages file for the distribution ... remove the pool
    sudo touch extras/binary-i386/Packages.gz && sudo chmod 666 extras/binary-i386/Packages.gz
    dpkg-scanpackages -m extras/binary-i386 | gzip -9c > extras/binary-i386/Packages.gz
#    sudo chmod 666 main/binary-i386/Packages.gz
#    dpkg-scanpackages -m main/binary-i386 | gzip -9c > main/binary-i386/Packages.gz
    popd

#    createFTPArchive
}

identifyPRPCMedia()
{
    echo "Identifying PRPC installation media directories - please wait..."

    # If identified as an ISO file, mount it to a temporary directory...
    if [ "Z" != "Z${prpc_mediadir}" ] && [ -f ${prpc_mediadir} ] ; then
	temp_dir=`mktemp -d -p ${prpcubudir}`
	sudo mount -o loop ${prpc_mediadir} ${temp_dir}
	export prpc_mediadir=${temp_dir}
    fi

    while [ ! -d ${prpc_mediadir}/schema ] ; do
	if [ "Z" != "Z${prpc_mediadir}" ] ; then
	    printf "  Invalid media directory - ${prpc_mediadir}:\n\
   subdirectory 'schema' does not exist ... searching for alternatives..."
	    find / -wholename */schema/*oracledate -type d -exec dirname {} \; | xargs -ifoo dirname foo | xargs -ibar dirname bar
	fi
	read -p "Enter the location of the PRPC media directory: " prpc_mediadir
    done
    if [ "Z${prpc_version}" = "Z" ] ; then
	read -p "Enter PRPC version / service pack level (no '-'  e.g. use 6.1sp1 NOT 6.1-sp1): " prpc_version
    fi
}

makePrpcInstaller()
{
    pi_sdir="$1"
    pi_tdir="${prpcubudir}/debian"

    if [ "Z${prpc_pkg}" = "Z" ] ; then
	prpc_pkg=${prpcubudir}/pegasystems-prpc_${prpc_version}_all.deb
    fi

    echo "Copying PRPC distribution into debian packaging directory..."
    prpc_idir=/usr/share/prpc-${prpc_version}
    mkdir -p "${pi_tdir}${prpc_idir}"
    rsync --delete -av "${pi_sdir}"/ "${pi_tdir}${prpc_idir}"
    # Prune out the ear files...
    find "${pi_tdir}${prpc_idir}" -name "*.ear" -exec rm -f {} \; 

    echo "Generating PRPC package metadata and script files..."
    mkdir -p "${pi_tdir}/DEBIAN"
    printf "Package: pegasystems-prpc\nVersion: $prpc_version\nSection: web\nPriority: optional\nArchitecture: all\nEssential: no\nDepends: sun-java6-jdk, tomcat6, oracle-xe\nPre-Depends: finger, wget, curl, sed\nRecommends: slapd, postfix\nInstalled-Size: 2048000\nMaintainer: ${dist_packaging_user} <${dist_packaging_email}>\nProvides: prpc\nDescription: PRPC (PegaRULES ProcessCommander) BPMS runtime and developer \n\tenvironment. Also includes documentationand required source to meet \n\tlicensing constraints on included open source technologies.\n PegaRULES ProcessCommander may be installed on any of a number of application \n\tserver environments to provide BPM capabilities. This Package installs \n\ton Tomcat6 and Orace10 XE on an Ubuntu system. The configuration \n\tfacility also configures an LDAP and PostFix environment as an \n\toptional step if those packages are installed in the OS configuration.\n" > "${pi_tdir}/DEBIAN/control"
    find "${pi_sdir}" -name \*icens*.txt -exec cat {} \; > "${pi_tdir}/DEBIAN/copyright"

    while [ "Z${prpc_dbtsname}" = "Z" ] ; do
	read -p "Enter the name of the DB tablespace: " prpc_dbtsname
    done
    while [ "Z${prpc_db_userid}" = "Z" ] ; do
	read -p "Enter the admin user name for the DB / Tomcat: " prpc_db_userid
    done
    if [ "Z${prpc_db_passwd}" = "Z" ] ; then
	prpc_db_passwd=Z; REPLY=X
        # Password capture should use the -s option and verify
	while [ "Z$REPLY" != "Z${prpc_db_passwd}" ]; do 
	    read -s -p "Enter the admin password for the DB / Tomcat: " prpc_db_passwd
	    read -s -p "Re-enter the admin password for the DB / Tomcat: "
	done;
    fi
    if [ "Z${ora_system_passwd}" = "Z" ] ; then
	ora_system_passwd=Z; REPLY=X
        # Password capture should use the -s option and verify
	while [ "Z$REPLY" != "Z${ora_system_passwd}" ]; do 
	    read -p -s "Enter the SYSTEM password for the Oracle DB: " ora_system_passwd
	    read -p -s "Re-enter the SYSTEM password for the Oracle DB: "
	done;
    fi

    # Extract install script file to postinst for inclusion in the package
    if [ -f "${pi_tdir}/DEBIAN/postinst" ] ; then rm -f "${pi_tdir}/DEBIAN/postinst"; fi
    tail -$(($ps_end - $pi_start + 1)) "$0" | head -$(($ps_start - $pi_start - 1)) | sed -e 's/<distname>/'${distname}'/g' -e 's/<prpc_version>/'${prpc_version}'/g' -e 's/<prpc_mediadir>/\/usr\/share\/prpc-'${prpc_version}'/g' -e 's/<prpc_userid>/'${prpc_db_userid}'/g' -e 's/<prpc_passwd>/'${prpc_db_passwd}'/g' -e 's/<ora_system_passwd>/'${ora_system_passwd}'/g' -e 's/<ora_appexp_port>/'${ora_appexp_port}'/g' -e 's/<prpc_dbtsname>/'${prpc_dbtsname}'/g' -e 's/<prpc_dbname>/'${prpc_dbname}'/g' > "${pi_tdir}/DEBIAN/postinst"

    # Extract removal script file to prerm for inclusion in the package
    if [ -f "${pi_tdir}/DEBIAN/prerm" ] ; then rm -f "${pi_tdir}/DEBIAN/prerm"; fi
    tail -$(($ps_end - $pr_start + 1)) "$0" | head -$(($pi_start - $pr_start - 1)) | sed -e 's/<prpc_dbtsname>/'$prpc_dbtsname'/g' -e 's/<prpc_dbname>/'$prpc_dbname'/g' -e 's/<prpc_userid>/'${prpc_db_userid}'/g' -e 's/<prpc_passwd>/'${prpc_db_passwd}'/g' -e 's/<ora_system_passwd>/'${ora_system_passwd}'/g' -e 's/<ora_appexp_port>/'${ora_appexp_port}'/g' -e 's/<prpc_version>/'${prpc_version}'/g' -e 's/<prpc_mediadir>/\/usr\/share\/prpc-'${prpc_version}'/g' > "${pi_tdir}/DEBIAN/prerm"

    # Copy ldif file into the prpc media directory
    sys_domain1=`echo ${sys_domain} | cut -f 1 -d '.'`
    sys_domain2=`echo ${sys_domain} | cut -f 2 -d '.'`
    tail -$(($ps_end - $pl_start + 1)) "$0" | head -$(($pbl_start - $pl_start - 1)) | sed -e 's/<ldapadmpasswd>/'${ldap_adm_passwd}'/g' -e 's/<domain1>/'${sys_domain1}'/g' -e 's/<domain2>/'${sys_domain2}'/g' > "${pi_tdir}${prpc_idir}/prpcbe.ldif"
    tail -$(($ps_end - $pbl_start + 1)) "$0" | head -$(($pr_start - $pbl_start - 1)) | sed -e 's/<prpc_passwd>/'${prpcadmin_os_passwd}'/g' -e 's/<domain1>/'${sys_domain1}'/g' -e 's/<domain2>/'${sys_domain2}'/g' > "${pi_tdir}${prpc_idir}/prpc.ldif"

    # TODO - review need for the conffiles, md5sums, etc...

    # Create contents of the debian package
    chmod 555 "${pi_tdir}/DEBIAN/postinst" "${pi_tdir}/DEBIAN/prerm"
    echo 2.0 > "${pi_tdir}/DEBIAN/debian-binary"

    # Create the debian package
    dpkg -b ${pi_tdir} ${prpc_pkg}

    if [ "no" = "${keep_prpc_wd}" ] ; then
	rm -rf ${pi_tdir}
    fi

    printf "\n\nPackaged installer created for PRPC as ${prpc_pkg}\n\n" 
}

confirmSettings() {
    read -e -i "$prpcadmin_os_password" -p "Set password for default user 'prpcadmin' as: " prpcadmin_os_password
    ldap_adm_passwd=${prpcadmin_os_passwd}
    read -e -i "$prpc_mediadir" -p "Confirm location of PRPC media: " prpc_mediadir
    read -e -i "$prpc_version" -p "Confirm PRPC version: " prpc_version
    read -e -i "$prpc_dbname" -p "Confirm database name: " prpc_dbname
    read -e -i "$prpc_dbtsname" -p "Confirm database tablespace name: " prpc_dbtsname
    read -e -i "$prpc_db_userid" -p "Confirm DB User ID: " prpc_db_userid
    read -e -i "$prpc_db_passwd" -p "Confirm DB password for ${prpc_db_uerid}: " prpc_db_passwd
    read -e -i "$ora_system_passwd" -p "Confirm SYSTEM and SYS password for oracle: " ora_system_passwd
    read -e -i "$ora_appexp_port" -p "Confirm oracle admin web application port: " ora_appexp_port
    read -e -i "$sys_domain" -p "Confirm system domain name: " sys_domain
    read -e -i "$sys_hostname" -p "Confirm system hostname: " sys_hostname
    read -e -i "$prpcubudir"  -p "Confirm distribution build working directory: " prpcubudir
    read -e -i "$dist_packaging_user" -p "Confirm distribution packaging user's name: " dist_packaging_user
    read -e -i "$dist_packaging_email" -p "Confirm distribution packaging user's e-mail: " dist_packaging_email
    read -e -i "$ubuntu_iso" -p "Confirm Ubuntu ISO location / save-to location: " ubuntu_iso
    read -e -i "$prpcbuntu_iso" -p "Confirm target location for generated PRPCbuntu ISO: " prpcbuntu_iso
    read -e -i "$oracle_deb" -p "Confirm Oracle 10g XE package location / save-to location: " oracle_deb
    read -e -i "$ubuntu_dist" -p "Confirm Ubuntu distribution flavor: " ubuntu_dist
    read -e -i "$ubuntu_bits" -p "Confirm Ubuntu distribution architecture bits: " ubuntu_bits
    read -e -i "$ubuntu_lang" -p "Confirm language (2 character identifier): " ubuntu_lang
    read -e -i "$ubuntu_locale" -p "Confirm country code (upper case only): " ubuntu_locale
    read -e -i "$ubuntu_conlayout" -p "Confirm console / keyboard layout: " ubuntu_conlayout
}

while getopts d:D:f:hH:im:o:O:p:P:S:T:u:U:v:V:x option
do
    case $option in
	d)
	    prpcbuntu_iso=$OPTARG
	    ;;
	D)
	    prpc_dbname=$OPTARG
	    ;;
	f)
	    vm_format=${OPTARG}
	    ;;
	H)
	    sys_hostname=${OPTARG}
	    ;;
	i)
	    prpcbuntu_confirm_settings=1
	    ;;
	m)
	    prpc_mediadir=${OPTARG}
	    ;;
	o)
	    oracle_deb=${OPTARG}
	    ;;
	O)
	    ora_system_passwd=${OPTARG}
	    ;;
	p)
	    prpc_pkg=${OPTARG}/pegasystems-prpc_${prpc_version}_all.deb
	    ;;
	P)
	    prpc_db_passwd=${OPTARG}
	    ;;
	S)
	    prpcadmin_os_passwd=${OPTARG}
	    ;;
	T)
	    prpc_dbtsname=$OPTARG
	    ;;
	u)
	    ubuntu_iso=${OPTARG}
	    ;;
	U)
	    prpc_db_userid=${OPTARG}
	    ;;
	v)
	    vm_img=${OPTARG}
	    ;;
	V)
	    prpc_version=${OPTARG}
	    ;;
	x)
	    prpcbuntu_iso_skip=1
	    ;;
	h)
	    printf "\nUsage: `basename $0` [OPTION]\n\n\
-d\tFilename for the target ISO image for Ubuntu install with PRPC \n\
\tand Oracle 10G Express Edition.\n\
-D\tDatabase name for PRPC installation - must previously exist - \n\
\tdefaults to XE\n\
-f\tSpecify format of vm image if -v is specified. Please refer to the\n\
\tdocumentation on qemu-img for information on formats. Values may be \n\
\tone of the following: raw, qcow2, vmdk\n\
-h\tPrint command help information\n\
-H\tHostname to be set on the target system\n\
-i\tInteractive mode - confirm all settings (requires readline)\n\
-m\tPath to the media or directory containing the PRPC distribution\n\
\tThis may be either a directory or an ISO image.\n\
-o\tLocation of the oracle debian package to use for the installation\n\
-O\tOracle system user password\n\n\
-p\tTarget directory into which the debian package created for the\n\
\tPRPC install will be written\n\
-P\tPassword for the database authentication for PRPC\n\
-S\tSystem level password for the prpcadmin user\n\n\
-T\tDB tablespace name for PRPC installation - will be created if it \n\
\tdoesn't exist\n\
-u\tLocation of the Ubuntu ISO image to use as the baseline for \n\
\tcreating PRPCbuntu\n\
-U\tUser name for database authentication for PRPC\n\
-v\tFilename for the target VM image into which PRPCbuntu will install\n\
-V\tThe version of PRPC to bundle - do not use the '-' character\n\
-x\tOnly create PRPC package - skip creation of the Ubuntu ISO\n\
\n\
This script file creates a Ubuntu distribution with automated installation \n\
and configuration of a fully functional PRPC environment leveraging an \n\
Oracle 10g XE RDBMS as the backend database and the native Tomcat install \n\
in a stock Ubunutu distribution as the J2EE application server. In addition, \n\
the partners repository will also be activated and the Sun JDK will be \n\
installed as part of the system configuration\n\
\n\
Creation of a PRPC VM can be completed in 5 easy steps after ensuring\n\
the PRPC software distribution is accessible from the shell environment:\n\n\
1.  Execute this script - accept most default settings  except the location \n\
    of the Ubuntu ISO and the PRPC distribution. The Ubuntu ISO location is \n\
    where it will be saved - PRPC is where the PRPC directory exists on \n\
    your system\n\
2.  Install PRPCbuntu to a clean VM using the prpcbuntu.iso image as the \n\
    CD / DVD to boot / install the system - it should be located in:\n\
        ~/tmp/prpcubu/prpcbuntu.iso\n\
4.  Accept all defaults going through the install\n\
5.  When complete, reboot with the same prpcbuntu.iso mapped as the CD drive\n\
6.  Log in as prpcadmin/changeme and execute the following command: \n\
        # sudo /etc/init.d/prpc-firstrun start\n\
\n\n\
Examples:\n\n\
To create only the PRPC debian installer in the current user's home directory\n\
from a dvd mounted to /mnt/dvd, skipping the Ubunut ISO creation:\n\n\
\t# sudo `basename $0` -x -p ~ -m /mnt/dvd\n\
\n\
"
	    exit 0
	    ;;
    esac
done

if [ ! -d ${prpcubudir} ] ; then mkdir -p ${prpcubudir}; fi

# Install needed supporting apps through native package manager...
pkg_util=`whereis apt-get | cut -f 2 -d ':' | cut -f 2 -d ' '`
if [ "Z${pkg_util}" = "Z" ] ; then
    # Install dependencies via yum package manager
    pkg_util=`whereis yum | cut -f 2 -d ':' | cut -f 2 -d ' '`
    sudo ${pkg_util} install dpkg dpkg-devel readline gnupg sed wget fakeroot rsync finger genisoimage > /dev/null
else
    # Install dependencies via aptitude package manager
    sudo ${pkg_util} install readline-common libreadline6 gnupg sed wget fakeroot rsync finger genisoimage > /dev/null
fi
    
read -e -i "${keep_prpc_wd}" -p "Should PRPC packaging working directory be saved? " keep_prpc_wd

if [ $prpcbuntu_confirm_settings -eq 1 ] ; then
    confirmSettings
fi

prpc_pkg=${prpcubudir}/pegasystems-prpc_${prpc_version}_all.deb

if [ ! -d ${prpcubudir} ] ; then mkdir -p ${prpcubudir}; fi

# Create the PRPC installer in ${prpcubudir} or where specified by options
identifyPRPCMedia
makePrpcInstaller "${prpc_mediadir}" 
# Remove mount if loopback - remove mountpoint if successful
sudo umount "${prpc_mediadir}" 2> /dev/null && rmdir "${prpc_mediadir}"
    
# Create the ISO image for a full system install
if [ $prpcbuntu_iso_skip -eq 0 ] ; then
    getAndExtractUbunutu

    # Change to the extracted image directory - simplifies scripting...
    pushd "${prpcubudir}/cd-image"
    
    # Identify the dist directory in the Release
    distname=`ls -1 -t dists/ | grep -v stable | head -1`
    extrasdir=${prpcubudir}/cd-image/dists/${distname}/extras/binary-i386/pool/extras

    # Create the extras directory for PRPC and Oracle
    sudo mkdir -m 777 -p ${extrasdir}
    
    pushd "${extrasdir}"
    if [ -f ${oracle_deb} ] ; then
	cp ${oracle_deb} .
    else
	wget --progress=dot:mega http://oss.oracle.com/debian/dists/unstable/non-free/binary-i386/oracle-xe_10.2.0.1-1.1_i386.deb
    fi
    popd

    cp -a ${prpc_pkg} "${extrasdir}"

    # Create the extras Release file...
    distversion=`grep Version\: dists/${distname}/Release | cut -f 2 -d ' '`
    sudo touch "${extrasdir}/../../Release" && sudo chmod 666 "${extrasdir}/../../Release"
    printf "Archive: ${distname}\nVersion: ${distversion}\nComponent: extras\nOrigin: Ubuntu\nLabel: Ubuntu\nArchitecture: i386" > ${extrasdir}/../../Release

    # Modify the boot config to default to PRPC Server referencing the new preseed file
    instmenucfg=`grep include isolinux/menu.cfg | cut -f 2 -d ' ' | xargs -ifoo grep -l -s default isolinux/foo`
    sudo sed -i.bak 's/default .*/default prpcsvr\ntimeout 10\nlabel prpcsvr\n  menu label ^Install PRPC Server\n  kernel \/install\/vmlinuz\n  append  file=\/cdrom\/preseed\/prpcsvr.seed locale='${ubuntu_lang}'_'${ubuntu_locale}' console-setup\/layoutcode='${ubuntu_conlayout}' localechooser\/translation\/warn-light=true localechooser\/translation\/warn-severe=true hostname='${sys_hostname}' domain='${sys_domain}' initrd=\/install\/initrd.gz ramdisk_size=16384 root=\/dev\/ram rw quiet --/g' ${instmenucfg}

    # Create / modify the preseed file
    sys_domain1=`echo ${sys_domain} | cut -f 1 -d '.'`
    sys_domain2=`echo ${sys_domain} | cut -f 2 -d '.'`
    tail -$(($ps_end - $ps_start + 1)) "$0" | sed -e 's/<prpc_password>/'$prpcadmin_os_password'/g' -e 's/<sys_hostname>/'$sys_hostname'/g' -e 's/<sys_domain>/'$sys_domain'/g' -e 's/<sys_domain1>/'$sys_domain1'/g' -e 's/<sys_domain2>/'$sys_domain2'/g' -e 's/<ldapadmpasswd>/'${ldap_adm_passwd}'/g' > ../prpcsvr.seed
    # PRPC install before v 6.1 requires running tomcat - disable install with
    # the OS package if version is not 6.1 or higher
    if [[ "${prpc_version}" < "6.1" ]] ; then
	sed -i 's/ pegasystems-prpc//g' ../prpcsvr.seed
        # ... enable the root login for access to the system if prpcadmin not
        # created due to not installing prpc
	sed -i 's/#\(.*\)passwd\/root-login\(.*\)/\1passwd\/root-login\2/g' ../prpcsvr.seed
    fi
    sudo mv ../prpcsvr.seed preseed 

    # Create firstrun script
    set +H # Turn off history expansion
    printf "#!/bin/bash\n### BEGIN INIT INFO\n# Provides:\tprpc-firstrun\n# Required-Start:\t\$local_fs \$remote_fs \$network \$syslog tomcat6\n# Required-Stop:\t\$local_fs \$remote_fs \$network \$syslog tomcat6\n# Should-Start:\tslapd postfix\n# Should-Stop:\tslapd postfix\n# Default-Start:\t2 3 4 5\n# Default-Stop:\t0 1 6\n# Short-Description:\tinitialize the system - load oracle and prpc after clean install on first run only\n### END INIT INFO\n# Source function library\n\nconfigureOpenLDAP() {\n\tldapadd -Y EXTERNAL -H ldapi:/// -f /etc/ldap/schema/cosine.ldif\n\tldapadd -Y EXTERNAL -H ldapi:/// -f /etc/ldap/schema/nis.ldif\n\tldapadd -Y EXTERNAL -H ldapi:/// -f /etc/ldap/schema/inetorgperson.ldif\n\tldapadd -Y EXTERNAL -H ldapi:/// -f ${prpc_idir}/prpcbe.ldif\n\tldapadd -Y EXTERNAL -H ldapi:/// -f ${prpc_idir}/prpc.ldif\n}\nstart() {\n}\ninstall() {\n\tmkdir /mnt/cdrom\n\tmount /dev/cdrom /mnt/cdrom\n\t#dpkg -i /mnt/cdrom/dists/${distname}/extras/binary-i386/pool/extras/oracle-xe*.deb\n\t#sed -i.bak 's/# \(deb.*partner\)/\\1/g' /etc/apt/sources.list\n\t#apt-get update\n\t#apt-get install sun-java6-jdk\n\t#/etc/init.d/tomcat6 stop\n\tdpkg -l pegasystems-prpc\n\t[ $? -gt 0 ] && dpkg -i /mnt/cdrom/dists/${distname}/extras/binary-i386/pool/extras/pegasystems-prpc*.deb\n\tchmod 444 \$0\n\tumount /mnt/cdrom\n\trmdir /mnt/cdrom\n\tconfigureOpenLDAP\n\treturn 0\n}\nstop() {\n\treturn 0\n}\nstatus() {\n\treturn 0\n}\nrestart() {\n\tstop\n\tstart\n\treturn \$?\n}\ncase \"\$1\" in\n\tinstall)\n\t\tinstall\n\t\tRETVAL=\$?\n\t\t;;\n\tstart)\n\t\tstart\n\t\tRETVAL=\$?\n\t\t;;\n\tstop)\n\t\tstop\n\t\tRETVAL=\$?\n\t\t;;\n\trestart)\n\t\trestart\n\t\tRETVAL=\$?\n\t\t;;\n\tstatus)\n\t\tstatus\n\t\tRETVAL=\$?\n\t\t;;\n\t*)\n\t\tprintf \"\n\nUsage: \$0 install|start|stop|restart|status\n\n\"\n\t\tRETVAL=\$?\n\t\t;;\nesac\nexit \$RETVAL\n" > ../prpc-firstrun
    chmod 777 ../prpc-firstrun && sudo mv ../prpc-firstrun prpc-firstrun

    #repackageSigningKeys
    repackageUbuntuPackages
    popd

    # Create target iso image to burn the CD
    sudo chmod a+w ${prpcubudir}/cd-image/isolinux/isolinux.bin
    mkisofs -r -V "PRPC Ubuntu Install CD" -cache-inodes -J -l -b isolinux/isolinux.bin -c isolinux/boot.cat -no-emul-boot -boot-load-size 4 -boot-info-table -o ${prpcbuntu_iso} ${prpcubudir}/cd-image
    sudo chmod a-w ${prpcubudir}/cd-image/isolinux/isolinux.bin
    printf "\n\nInstallation image successfully created as ${prpcbuntu_iso}\n\n"

    if [ "Z${vm_img}" != "Z" ] ; then
	if [ ! -f ${vm_img} ] ; then
	    if [ "Z${vm_format}" != "Z" ] ; then
		qemu-img create -f ${vm_format} ${vm_img} 12G
	    fi
	fi
	
        # Create / install on the vm_img if one is specified with -v
	if [ -f ${vm_img} ] ; then
	    qemu-kvm -daemonize -name prpc${prpc_version} -m 1536 -smp 2 -no-reboot -boot d -drive file=${vm_img},if=ide,bus=0,unit=0,boot=on,format=${vm_format} -drive file=${prpcbuntu_iso},if=ide,media=cdrom,bus=1,unit=0,format=raw -usb -vnc localhost:6 -k en-us -vga cirrus -soundhw es1370 -redir tcp:48080::8080 -redir tcp:40022::22
	    vncviewer localhost:6
	    printf "\nWe will now restart the VM to finish the install.\n\
Login as prpcadmin/${prpcadmin_os_password} after the restart\n\
and execute the following command:\n\n\
\tsudo /etc/init.d/prpc-firstrun start\n\nPress enter to start the VM\n"
	    read
	    qemu-kvm -daemonize -name prpc${prpc_version} -m 1536 -smp 2 -no-reboot -boot c -drive file=${vm_img},if=ide,bus=0,unit=0,boot=on,format=${vm_format} -drive file=${prpcbuntu_iso},if=ide,media=cdrom,bus=1,unit=0,format=raw -usb -vnc localhost:6 -k en-us -vga cirrus -soundhw es1370 -redir tcp:48080::8080 -redir tcp:40022::22
            # Note - firstrun could be automated through ssh when reboot is done
	    vncviewer localhost:6
	fi
    fi
fi

exit 0

###############################################################################
### DO NOT CHANGE ANY '### START XXX' LINES FOLLOWING THIS LINE ###############
###############################################################################
### START PRPC LDIF INCLUDE ###
dn: dc=<domain1>,dc=<domain2>
objectClass: dcObject
objectClass: organization
o: Example Co
dc: <domain1>

dn: cn=ldapadmin,dc=<domain1>,dc=<domain2>
objectClass: organizationalRole
cn: ldapadmin
roleOccupant: cn=PRPC Administrator,dc=<domain1>,dc=<domain2>
roleOccupant: cn=ldapadm,dc=<domain1>,dc=<domain2>

dn: cn=PRPC Administrator,dc=<domain1>,dc=<domain2>
cn: PRPC Administrator
objectClass: inetOrgPerson
objectClass: person
objectClass: organizationalPerson
objectClass: posixAccount
givenName: PRPC
sn: Administrator
userPassword: <prpc_passwd>
o: Example Co
uid: prpcadmin2
uidNumber: 1002
gidNumber: 1002
loginShell: /bin/bash
homeDirectory: /home/prpcadmin2
mail: prpcadmin2@<domain1>.<domain2>

### START PRPC BE LDIF INCLUDE ###
# Load dynamic backend modules
dn: cn=module,cn=config
objectClass: olcModuleList
cn: module
olcModulepath: /usr/lib/ldap
olcModuleload: back_hdb

# Database settings
dn: olcDatabase=hdb,cn=config
objectClass: olcDatabaseConfig
objectClass: olcHdbConfig
olcDatabase: {1}hdb
olcSuffix: dc=<domain1>,dc=<domain2>
olcDbDirectory: /var/lib/ldap
olcRootDN: cn=ldapadm,dc=<domain1>,dc=<domain2>
olcRootPW: <ldapadmpasswd>
olcDbConfig: set_cachesize 0 2097152 0
olcDbConfig: set_lk_max_objects 1500
olcDbConfig: set_lk_max_locks 1500
olcDbConfig: set_lk_max_lockers 1500
olcDbIndex: objectClass eq
olcLastMod: TRUE
olcDbCheckpoint: 512 30
olcAccess: to attrs=userPassword by dn="cn=ldapadm,dc=<domain1>,dc=<domain2>" write by anonymous auth by self write by * none
olcAccess: to attrs=shadowLastChange by self write by * read
olcAccess: to dn.base="" by * read
olcAccess: to * by dn="cn=ldapadm,dc=<domain1>,dc=<domain2>" write by * read

### START PRPC REMOVE INCLUDE ###
#!/bin/bash
prpc_version=<prpc_version>
prpc_mediadir="<prpc_mediadir>"
prpc_dbtsname=<prpc_dbtsname>
prpc_userid=<prpc_userid>
prpc_passwd=<prpc_passwd>
ora_system_passwd=<ora_system_passwd>

RETVAL=0
if [ -h /usr/share/doc/prpc-${prpc_version} ] ; then
    rm "/usr/share/doc/prpc-${prpc_version}"
    RETVAL=$?
fi
if [ -f ${prpc_mediadir}/install_script.sh ] ; then
    rm ${prpc_mediadir}/install_script.sh
    RETVAL=$?
fi
if [ -d ${HOME}/pega_temp ] ; then
    rm -rf ${HOME}/pega_temp
fi

# Should the DB be archived???

exit $RETVAL

### START PRPC INSTALL INCLUDE ###
#!/bin/bash
# Global parameter defines
prpc_version=<prpc_version>
prpc_mediadir="<prpc_mediadir>"
prpc_dbhost=`hostname`
prpc_dbname=<prpc_dbname>
prpc_dbtsname=<prpc_dbtsname>
prpc_userid=<prpc_userid>
prpc_passwd=<prpc_passwd>
ora_appexp_port=<ora_appexp_port>
ora_system_passwd=<ora_system_passwd>
prpcinstall_confirm_settings=0

escape_dirname()
{
    fdn=$1
    dn="\/"`basename ${fdn}`
    fdn=`dirname ${fdn}`
    while [ "/" != "${fdn}" ] ; do
	dn="\/"`basename ${fdn}`"${dn}"
	fdn=`dirname ${fdn}`
	echo dn is $dn - fdn is $fdn 1>&2
    done;
    echo $dn
}

rhel_checkSupportingServices()
{
    rpm -q openldap openldap-servers openldap-clients postfix tomcat6
    if [ $? -eq 1 ] ; then
      yum install openldap openldap-servers openldap-clients postfix tomcat6
      read -p "openldap and postfix reconfiguration required, press <enter> to continue"
    fi
}

ubu_checkSupportingServices()
{
    dpkg -l slapd ldap-utils postfix tomcat6 > /dev/null
    if [ $? -eq 1 ] ; then
      apt-get install slapd ldap-utils postfix tomcat6
      dpkg-reconfigure slapd
      dpkg-reconfigure postfix
    fi
}

rhel_installSunJDK()
{
    read -p "Sun JDK install automation not yet supported, press <enter> to continue"
}

ubu_tuneOS() 
{
    printf "Setting kernel runtime parameters for oracle performance...\n"
    # Edits for /etc/sysctl for oracle
    #fs.file-max = 65535
    #kernel.shmall = 2097152
    #kernel.shmmax = 2147483648
    #kernel.shmmni = 4096
    #kernel.sem = 250 32000 100 128
    #net.ipv4.ip_local_port_range = 1024 65535
    #net.core.rmem_default = 1048576
    #net.core.rmem_max = 1048576
    #net.core.wmem_default = 262144
    #net.core.wmem_max = 262144

    printf "Setting kernel limits for oracle performance...\n"
    # Edits for /etc/security/limits.conf for oracle
    #oracle soft nproc 2047
    #oracle hard nproc 16383
    #oracle soft nofile 1023
    #oracle hard nofile 65535

    # Reload sysctl settings...
    sysctl -p
}

ubu_installSunJDK()
{
    dpkg -l sun-java6-jdk > /dev/null
    if [ $? -gt 0 ] ; then
        # Enable partner software sources for apt
	printf "\nUpdating apt catalogue to install Sun Java 6 JDK from partner sources online...\n\n"
	sleep 3

        #sed -i.bak 's/# \(deb.*partner\)/\1/g' /etc/apt/sources.list
	distname=`lsb_release -c -s`
	sudo add-apt-repository "deb http://archive.canonical.com/ ${distname} partner"
	apt-get update
	apt-get install sun-java6-jdk
    fi
}

rhel_installOracle()
{
    rpm -q oracle-xe
    if [ $? -eq 1 ] ; then
	printf "Installing Oracle 10 XE:\n"
	wget  http://download.oracle.com/otn/linux/oracle10g/xe/10201/oracle-xe-10.2.0.1-1.0.i386.rpm
	#yum install bc libaio1 libc6 libstdc++5
	# rpm -i oracle-xe should take care of previous dependencies
	rpm -i oracle-xe_10.2.0.1-1.1_i386.rpm
	rm -f oracle-xe_10.2.0.1-1.1_i386.rpm
    fi
    ora_init_script=`rpm -q -l oracle-xe | grep /etc/init.d/`
    ora_answers_tmp=`mktemp`
    printf "${ora_appexp_port}\n\n${ora_system_passwd}\n${ora_system_passwd}\ny\n" > ${ora_answers_tmp}

    sed -i.bak 's/8080/'${ora_appexp_port}'/g' /etc/sysconfig/oracle-xe
    sed -i 's/CONFIGURE_RUN=true/CONFIGURE_RUN=false/g' /etc/sysconfig/oracle-xe
    ${ora_init_script} configure < ${ora_answers_tmp}
    rm ${ora_answers_tmp}
}

ubu_installOracle()
{
    # Install oracle-xe if not present
    dpkg -l oracle-xe > /dev/null
    if [ $? -gt 0 ] ; then
	printf "Installing Oracle 10 XE:\n"
	wget  http://oss.oracle.com/debian/dists/unstable/non-free/binary-i386/oracle-xe_10.2.0.1-1.1_i386.deb
	#apt-get install bc libaio libaio1 libc6 libstdc++5
	# dpkg -i oracle-xe should take care of previous dependencies
	dpkg -i oracle-xe_10.2.0.1-1.1_i386.deb
	rm -f oracle-xe_10.2.0.1-1.1_i386.deb
    fi
    ora_init_script=`dpkg -L oracle-xe | grep /etc/init.d/`

    sed -i.bak 's/#!\/.*/\0\n### BEGIN INIT INFO\n# Provides:\toracle_xe\n# Required-Start:\t$local_fs $remote_fs $network $syslog\n# Required-Stop:\t$local_fs $remote_fs $network $syslog\n# Default-Start:\t2 3 4 5\n# Default-Stop:\t0 1 6\n# Short-Description:\tOracle 10g XE RDBMS (Relational Database Management System)\n### END INIT INFO\n/g' ${ora_init_script} && update-rc.d oracle-xe defaults

    printf "${ora_appexp_port}\n\n${ora_system_passwd}\n${ora_system_passwd}\ny\n" > /tmp/oracfg.txt

    sed -i.bak 's/8080/8191/g' /etc/default/oracle-xe
    sed -i 's/CONFIGURE_RUN=true/CONFIGURE_RUN=false/g' /etc/default/oracle-xe
    ${ora_init_script} configure < /tmp/oracfg.txt
    rm /tmp/oracfg.txt

    ubu_tuneOS
}

configureOracle()
{
    while [ "Z$prpc_dbtsname" = "Z" ] ; do
	read -p "Enter the name of the DB tablespace: " prpc_dbtsname
    done
    while [ "Z$prpc_userid" = "Z" ] ; do
	read -p "Enter the admin user name for the DB / Tomcat: " prpc_userid
    done
    while [ "Z$prpc_passwd" = "Z" ] ; do
	read -p "Enter the admin password for the DB / Tomcat: " prpc_passwd
    done
    while [ "Z$ora_system_passwd" = "Z" ] ; do
	read -p "Enter the SYSTEM password for the Oracle DB: " ora_system_passwd
    done

    pkg_file_list=`mktemp`
    dpkg -L oracle-xe > ${pkg_file_list}
    if [ $? -eq 2 ] ; then
	rpm -q -l oracle-xe > ${pkg_file_list}
    fi

    ORA_OWNER=oracle
    ORACLE_SID=${prpc_dbname} ; export ORACLE_SID 
    OJDBC=`grep ojdbc14.jar ${pkg_file_list}` ; export OJDBC
    ORACLE_HOME=`grep server ${pkg_file_list} | grep -v server\/` ; export ORACLE_HOME
    # Quick and dirty - use the existing DB instance - fewer changes
    SQLPLUS=`grep bin\/sqlplus ${pkg_file_list}` ; export SQLPLUS

    pkg_file_list=`mktemp`
    dpkg -L tomcat6-common > ${pkg_file_list}
    if [ $? -eq 2 ] ; then
	rpm -q -l tomcat6-common > ${pkg_file_list}
    fi

    TC_BASE_LIB=`grep 6\/lib ${pkg_file_list} | grep -v lib\/` ; export TC_BASE_LIB

    rm ${pkg_file_list}

    cp $OJDBC $TC_BASE_LIB
    
    # TODO - Create the database indicated by ${prpc_dbname} if it does not exist

    # Assume the largest jar contains the application rule base - get the size
    prpc_rulejar=`find ${prpc_mediadir} -name PRPC\*.jar -exec du -m {} \; | sort -nr | head -n 1 | cut -f 2 -d '	'`
    prpc_rulejar_sz=`find ${prpc_mediadir} -name PRPC\*.jar -exec du -m {} \; | cut -f 1 | sort -nr | xargs echo | cut -f 1 -d ' '`
    create_db_sql=`mktemp`
    printf "CREATE BIGFILE TABLESPACE ${prpc_dbtsname} DATAFILE '"${prpc_dbtsname}".dbf' SIZE "$((${prpc_rulejar_sz} * 2 / 128 * 128 ))"M AUTOEXTEND on NEXT 32M MAXSIZE "$((${prpc_rulejar_sz} * 4 / 128 * 128 ))"M;\n\
CREATE USER "${prpc_userid}" IDENTIFIED BY \""${prpc_passwd}"\" DEFAULT TABLESPACE "${prpc_dbtsname}";\n\
CREATE ROLE prpcadmin;\n\
GRANT dba TO prpcadmin;\n\
GRANT prpcadmin TO "${prpc_userid}";\n\
ALTER USER "${prpc_userid}" QUOTA UNLIMITED ON "${prpc_dbtsname}";\n\
COMMIT;\n\
QUIT;\n" > ${create_db_sql}
    chown $ORA_OWNER ${create_db_sql}
    printf "\nCreating tablespace ${prpc_dbtsname}...\n\n"
    su -s /bin/bash -p $ORA_OWNER -c "$SQLPLUS -s / AS SYSDBA @${create_db_sql}"
    rm -f ${create_db_sql}
}

rhel_installDB2()
{
    read -p "DB2 Installation not yet supported, press <enter> to continue"
}

ubu_installDB2()
{
    read -p "DB2 Installation not yet supported, press <enter> to continue"
}

configureDB2()
{
    read -p "DB2 automated configuration not yet supported, press <enter> to continue"
}

addAdminUser()
{
    grep prpcadmin /etc/passwd > /dev/null
    if [ $? -gt 0 ] ; then
        # Could use crypt on a passwd with the -p option if we install mcrypt
	printf "Creating admin user prpcadmin as a sudo capable user...\n"
	/usr/sbin/useradd -d /home/prpcadmin -s /bin/bash -m -G sudo -c "PRPC Administrator" prpcadmin
	/usr/bin/printf "${prpc_passwd}\n${prpc_passwd}\n" | passwd prpcadmin
    fi
}

runRuleDBInstallServlet()
{
    printf "\nConfiguring tables in tablespace ${prpc_dbtsname}...\n\n"
    prpc_install_sql=`find ${prpc_mediadir}/schema -wholename *oracledate/*oracle*_insta* -print`
    echo "quit;" >> ${prpc_install_sql}
    $SQLPLUS ${prpc_userid}/${prpc_passwd} @${prpc_install_sql}

    # Make temp cookie file
    temp_cf=`mktemp`
    # Verify the database
    curl -c ${temp_cf} http://localhost:${tc_svr_port_dist}/prdbutil/FirstSteps/FirstStepsChkDB.jsp
    # Shutdown /prweb
    curl --user ${prpc_userid}:${prpc_passwd} http://localhost:${tc_svr_port_dist}/manager/stop?path=/prweb
    # Upload the rules file
    curl -c ${temp_cf} -F file=@"${prpc_rulejar}" http://localhost:${tc_svr_port_dist}/prdbutil/Upload
    # Run the rules import mechanism
    [ $? -eq 0 ] && curl -c ${temp_cf} -F isAppBundle=true -F ProcessType= -F fileName=`basename "${prpc_rulejar}"` http://localhost:${tc_svr_port_dist}/prdbutil/Importer
    # Index the rules for searching
    [ $? -eq 0 ] && mkdir /home/prpcadmin/rules_index && chmod 777 /home/prpcadmin/rules_index && curl -c ${temp_cf} -F idxHost=`hostname` -F idxDir=/home/prpcadmin/rules_index -F idxSoap=http://`hostname`:${tc_svr_port_dist}/prweb/PRSOAPServlet -F fileName=FirstSteps/FirstStepsIndexConfig.jar -F isDbUtilFile=true -F subProps=  http://localhost:${tc_svr_port_dist}/prdbutil/Importer
    # Setup help and sytem admin console
    [ $? -eq 0 ] && curl -c ${temp_cf} -F helpUri=http://`hostname`:${tc_svr_port_dist}/prhelp -F sysMgmtUri=http://`hostname`:${tc_svr_port_dist}/prsysmgmt -F fileName=FirstSteps/FirstStepsIndexConfig.jar -F isDbUtilFile=true -F subProps=  http://localhost:${tc_svr_port_dist}/prdbutil/Importer
    # Restart /prweb
    curl --user ${prpc_userid}:${prpc_passwd} http://localhost:${tc_svr_port_dist}/manager/start?path=/prweb
    # Remove cookie file if successful
    [ $? -eq 0 ] && rm -f ${temp_cf}

    # If not successful prompt user to tun prdbutil to load / index rules
    if [ ! -f ${temp_cf} ] ; then
	printf "Auto-configuration of prdbutil succeeded\n"
    else
	printf "Load PRPC using prdbutil through a web browser on the VM host:\n\
( http://"`hostname`":${tc_svr_port_dist}/prdbutil/index.html ) - values are:\n\
  DB Driver Class = oracle.jdbc.OracleDriver\n\
  DB Driver JAR = $OJDBC\n\
  DB Type = oracledate\n\
  DB Connect URL = ${prpc_connect}\n\
  DB User ID = ${prpc_userid}\n\
  DB User Password = ${prpc_passwd}\n\n"

	read -p "Select enter when done" db_config
	rm -f ${temp_cf}
    fi
}

runRuleDBInstallScript()
{
    echo "Checking for script based installer..."
    dbcfg_pf=`find ${prpc_mediadir} -name setupDatabase.properties`
    dbcfg_script=`dirname "${dbcfg_pf}"`/install.sh
    if [ -f "${dbcfg_script}" ] ; then
	echo "Scripted DB config found - executing..."
	#sed -i 's/^pega\.jdbc\.driver\.jar=.*/^pega\.jdbc\.driver\.jar='$OJDBC'/g' "${dbcfg_pf}"
	#sed -i 's/^pega\.jdbc\.driver\.jar=.*/^pega\.jdbc\.driver\.class=oracle.jdbc.OracleDriver/g' "${dbcfg_pf}"
	#sed -i 's/^pega\.database\.type=.*/^pega\.database\.type=oracledate/g' "${dbcfg_pf}"
	#sed -i 's/^pega\.jdbc\.url=.*/^pega\.jdbc\.url='${prpc_connect}'/g' "${dbcfg_pf}"
	#sed -i 's/^pega\.jdbc\.username=.*/^pega\.jdbc\.username='${prpc_userid}'/g' "${dbcfg_pf}"
	#sed -i 's/^pega\.jdbc\.password=.*/^pega\.jdbc\.password='${prpc_passwd}'/g' "${dbcfg_pf}"

	# Set and verify JAVA_HOME
	JAVA_HOME=/usr/lib/jvm/java-6-sun ; export JAVA_HOME
	# If JAVA_HOME does not exist take the most recently installed JVM
	if [ ! -d ${JAVA_HOME} ] ; then
	    jhome=`ls -t /usr/lib/jvm | xargs echo | cut -f 1 -d ' '`
	    while [ ! -d ${JAVA_HOME} ] ; do
	    	read -e -i ${jhome} -p "Confirm JAVA_HOME (for Tomcat and PRPC DB configuration): " JAVA_HOME ; export JAVA_HOME
	    done
	fi

	sed -i.bak 's/.*JAVA_HOME=.*/JAVA_HOME=\/usr\/lib\/jvm\/'`basename $JAVA_HOME`'/g' /etc/default/tomcat6

	pushd `dirname ${dbcfg_script}`
	sed -i.bak 's/#!\/.*/\0\nJAVA_HOME=\/usr\/lib\/jvm\/'`basename $JAVA_HOME`' ; export JAVA_HOME/g' ${dbcfg_script}
	chmod 755 ${dbcfg_script} bin/*
	"${dbcfg_script}" --driverClass oracle.jdbc.OracleDriver --driverJAR $OJDBC --dbType oracledate --dbURL ${prpc_connect} --dbUser ${prpc_userid} --dbPassword ${prpc_passwd}
	mv ${dbcfg_script}.bak ${dbcfg_script}
	popd
    else
	echo "Java installer not found - rule install must be completed manually"
    fi
}

linkTomcatAdminApps()
{
    dpkg -l tomcat6-admin > /dev/null
    if [ $? -eq 0 ] ; then
	ln -s /usr/share/tomcat6-admin/manager ${tc_deploydir}
	ln -s /usr/share/tomcat6-admin/host-manager ${tc_deploydir}
    fi    
}

deployPlatformWarFiles()
{
    # Deploy war files - soft link if curl fails (initial OS install in-target)
    helpwar=`find ${prpc_mediadir}/../ -name prhelp.war -print`

    # Stage and force load / deployment of the mgmt, app, and help war files
    # --- Soft links work best - deploy sometimes gets confused if the context
    #     has already been set in server.xml...
    chmod a+r "${prpc_mediadir}/archives/prsysmgmt.war"
    #curl --user ${prpc_userid}:${prpc_passwd} --upload-file "${prpc_mediadir}/archives/prsysmgmt.war" http://localhost:${tc_svr_port_dist}/manager/deploy
    #[ $? -gt 0 ] && 
    ln -s "${prpc_mediadir}/archives/prsysmgmt.war" ${tc_deploydir}

    chmod a+r "${prpc_mediadir}/archives/prweb.war"
    #curl --user ${prpc_userid}:${prpc_passwd} --upload-file "${prpc_mediadir}/archives/prweb.war" http://localhost:${tc_svr_port_dist}/manager/deploy
    #[ $? -gt 0 ] && 
    ln -s "${prpc_mediadir}/archives/prweb.war" ${tc_deploydir}

    chmod a+r "${helpwar}"
    #curl --user ${prpc_userid}:${prpc_passwd} --upload-file "${helpwar}" http://localhost:${tc_svr_port_dist}/manager/deploy
    #[ $? -gt 0 ] && 
    ln -s "${helpwar}" ${tc_deploydir}


    # Stage and force load / deployment of the prdbutil war
    if [ -f ${prpc_mediadir}/archives/prdbutil.war ] ; then
	chmod a+r "${prpc_mediadir}/archives/prdbutil.war"
	#curl --user ${prpc_userid}:${prpc_passwd} --upload-file "${prpc_mediadir}/archives/prdbutil.war" http://localhost:${tc_svr_port_dist}/manager/deploy
	#[ $? -gt 0 ] && 
	ln -s "${prpc_mediadir}/archives/prdbutil.war" ${tc_deploydir}
    fi

    # Force tomcat restart
    #${tc_script} restart

    # Remove the war files after deployment
    [ ! -h ${tc_deploydir}/prweb.war ] && sleep 30
    [ ! -h ${tc_deploydir}/prweb.war ] && rm -f ${tc_deploydir}/prweb.war
    [ ! -h ${tc_deploydir}/prhelp.war ] && rm -f ${tc_deploydir}/prhelp.war
    [ ! -h ${tc_deploydir}/prsysmgmt.war ] && rm -f ${tc_deploydir}/prsysmgmt.war
    [ -f ${tc_deploydir}/prdbutil.war ] && [ ! -h ${tc_deploydir}/prdbutil.war ] && rm -f ${tc_deploydir}/prdbutil.war
}

prepareTomcat()
{
    # Add LSB parameters ...Configure JVM parameters... turn off tomcat security
    printf "\nAdding LSB parameters to tomcat init script (saving .bak)...\n\n"
    sed -i.bak 's/#!\/.*/\0\n### BEGIN INIT INFO\n# Provides:\ttomcat6\n# Required-Start:\t$local_fs $remote_fs $network $syslog oracle_xe\n# Required-Stop:\t$local_fs $remote_fs $network $syslog oracle_xe\n# Should-Start:\tslapd postfix\n# Should-Stop:\tslapd postfix\n# Default-Start:\t2 3 4 5\n# Default-Stop:\t0 1 6\n# Short-Description:\tTomcat6 J2EE compliant servlet engine\n### END INIT INFO\n/g' ${tc_script}
    printf "\nTuning JVM memory and GC settings for PRPC...\n\n"
    sed -i 's/-Xmx[^ "]*/-Xmx1024M -Xms512M -XX:PermSize=64m -XX:MaxPermSize=256m -Doracle.jdbc.V8Compatible=true/g' ${tc_script}
    printf "\nTurning off security in tomcat init script...restarting tomcat...\n\n"
    sed -i 's/TOMCAT6_SECURITY=.*/TOMCAT6_SECURITY=no/g' ${tc_script} &&  update-rc.d `basename ${tc_script}` defaults

    # Configure app server user security... and restart
    export tc_user_conf=${tc_confdir}/tomcat-users.xml
    printf "\nAdding ${prpc_userid} to tomcat managers...\n\n"
    sed -i.bak 's/<\/tomcat-users>/  <role rolename="admin"\/>\n  <role rolename="manager"\/>\n  <user username="'${prpc_userid}'" password="'${prpc_passwd}'" roles="admin,manager"\/>\n<\/tomcat-users>/g' ${tc_user_conf}
    ${tc_script} restart

    # Configure DNS lookups...
    
    printf "Create: Conf=%s Script=%s DeployDir=%s\n" $tc_confdir $tc_script $tc_deploydir

    # Configure database as a global resource...
    echo Home directory is ${HOME}
    #ESCHOME=`escape_dirname ${HOME}`
    ESCHOME=`escape_dirname /home/prpcadmin`
    tc_svr_port_dist=`sed -e 's/.*port="\([0-9]*\)".*HTTP\/1\.1"/\1/g' -e 't' -e 'd'  ${tc_confdir}/server.xml | grep -v SSL | xargs -l echo | cut -f 1 -d ' '`
    # Add resource configurations for Oracle and bind to each context...
    sed -i.bak 's/\(.*\)<\/GlobalNamingResources>/\1\t<Environment name="simpleValue" type="java\.lang\.Integer" value="30"\/>\n\1\t<Resource name="jdbc\/PegaRULES" auth="Container" type="javax\.sql\.DataSource" username="'${prpc_userid}'" password="'${prpc_passwd}'" driverClassName="oracle\.jdbc\.OracleDriver" maxActive="50" maxIdle="5" maxWait="5000" url="'${prpc_connect}'" \/>\n\1<\/GlobalNamingResources>/g' ${tc_confdir}/server.xml
    sed -i 's/"localhost"/"'`hostname`'"/g' ${tc_confdir}/server.xml
    # Changing server.xml directly - no assurance server can run to capture
    # deployment XML on install - especially in PRPCbuntu distribution gen.
    # ... configure /prweb context
    sed -i 's/\(.*\)<\/Host>/\1\t<Context path="\/prweb">\n\1\t\t<Environment description="PegaRULES Temporary Directory" name="pega.tmpdir" type="java\.lang\.String" value="'${ESCHOME}'\/pega_temp"\/>\n\1\t\t<Resource auth="Container" description="File Access via a Resource Adaptor" name="eis\/PRAdapterConnectionFactory" type="javax\.resource\.cci\.ConnectionFactory"\/>\n\1\t\t<Resource auth="Container" description="Database-based File Access" name="jdbc\/PRFileStore" type="javax\.sql\.DataSource"\/>\n\1\t\t<Resource auth="Container" description="location of prlogging\.xml file" name="url\/pegarules\.logging\.configuration" type="java\.net\.URL"\/>\n\1\t\t<ResourceLink global="jdbc\/PegaRULES" name="jdbc\/PegaRULES" type="javax\.sql\.DataSource"\/>\n\1\t<\/Context>\n\1<\/Host>/g' ${tc_confdir}/server.xml
    # ... configure /prsysmgmt context
    sed -i 's/\(.*\)<\/Host>/\1\t<Context path="\/prsysmgmt">\n\1\t<\/Context>\n\1<\/Host>/g' ${tc_confdir}/server.xml
    # ... configure /prhelp context
    sed -i 's/\(.*\)<\/Host>/\1\t<Context path="\/prhelp">\n\1\t<\/Context>\n\1<\/Host>/g' ${tc_confdir}/server.xml
    if [ -f ${prpc_mediadir}/archives/prdbutil.war ] ; then
        # ... configure /prdbutil context
	sed -i 's/\(.*\)<\/Host>/\1\t<Context path="\/prdbutil">\n\1\t\t<ResourceLink global="jdbc\/PegaRULES" name="jdbc\/PegaRULES" type="javax\.sql\.DataSource"\/>\n\1\t<\/Context>\n\1<\/Host>/g'  ${tc_confdir}/server.xml
    fi

    printf "Deploy: Conf=%s Script=%s DeployDir=%s\n" $tc_confdir $tc_script $tc_deploydir
    deployPlatformWarFiles

    mkdir ${HOME}/pega_temp
    chmod 777 ${HOME}/pega_temp

    # This may be unnecessary if the default config picks up the tomcat server
    #  environment config chnages....
#    prconfig_files=`find -name prconfig.xml -print`
#    for f in ${prconfig_files} ; do
#        # Configure the temp directory
#	sed -i.bak 's/\(.*\)${pega.tmpdir}\(.*\)/\1'${ESCHOME}'\/pega_temp\2/g' $f
#    done

    # If the tomcat6-admin package is installed - ensure apps are linked to 
    #   webapps / deployment directory
    linkTomcatAdminApps

    $tc_script restart
}

setPrpcMediaDir()
{
    echo "Identifying PRPC installation media directories - please wait..."
#    prpc_mediadir=`find /usr/share/ -wholename */schema/*oracledate -type d -exec dirname {} \;`

    while [ ! -d ${prpc_mediadir}/schema ] ; do
	if [ "Z" != "Z${prpc_mediadir}" ] ; then
	    echo "  ${prpc_mediadir}/schema does not exist"
	fi
	read -p "Enter the location of the PRPC media directory: " prpc_mediadir
    done
    export prpc_mediadir
    if [ -d "${prpc_mediadir}/Documentation" ] ; then
	ln -s "${prpc_mediadir}/Documentation" /usr/share/doc/prpc-${prpc_version}
    fi
}

confirmInstallSettings()
{
    read -e -i ${prpc_version} -p "Confirm PRPC Version: " prpc_version
    read -e -i ${prpc_mediadir} -p "Confirm PRPC install media directory: " prpc_mediadir
    read -e -i ${prpc_dbhost} -p "Confirm host for install: " prpc_dbhost
    read -e -i ${prpc_dbname} -p "Confirm database name in database: " prpc_dbtsname
    read -e -i ${prpc_dbtsname} -p "Confirm tablespace name in database: " prpc_dbtsname
    read -e -i ${prpc_userid} -p "Confirm database user id: " prpc_userid
    read -e -i ${prpc_passwd} -p "Confirm database user password: " prpc_passwd
    read -e -i ${ora_system_passwd} -p "Confirm database system password: " ora_system_passwd
}

while getopts D:him:P:T:U:V: option
do
    case $option in
	D)
	    prpc_dbname=$OPTARG
	    ;;
	i)
	    prpcinstall_confirm_settings=1
	    ;;
	m)
	    prpc_mediadir=${OPTARG}
	    ;;
	P)
	    prpc_db_passwd=${OPTARG}
	    ;;
	T)
	    prpc_dbtsname=$OPTARG
	    ;;
	U)
	    prpc_db_userid=${OPTARG}
	    ;;
	V)
	    prpc_version=${OPTARG}
	    ;;
	h)
	    printf "\nUsage: `basename $0` [OPTION]\n\n\
-D\tDatabase name for PRPC installation - must previously exist - \n\
\tdefaults to XE\n\
-h\tPrint command help information\n\
-i\tInteractive mode - confirm all settings (requires readline)\n\
-m\tPath to the media or directory containing the PRPC distribution\n\
-P\tPassword for the database authentication for PRPC\n\
-T\tDB tablespace name for PRPC installation - will be created if it \n\
\tdoesn't exist\n\
-U\tUser name for database authentication for PRPC\n\
-V\tThe version of PRPC being installed\n\
\n\
This script file fully automates installation and configuration of a fully \n\
functional PRPC environment leveraging an Oracle 10g XE RDBMS as the backend \n\
database and the native Tomcat install in a stock Ubunutu distribution as \n\
the J2EE application server. In addition, the partners repository may also \n\
be activated and the Sun JDK will be installed as part of the system \n\
configuration\n\
\n\n\
Examples:\n\n\
None\n\
\n\
"
	    exit 0
	    ;;
    esac
done

if [ $prpcinstall_confirm_settings -eq 1 ] ; then
    confirmInstallSettings
fi

cp $0 ${prpc_mediadir}/install_script.sh

addAdminUser

printf "Verifying support services...\n"
ubu_checkSupportingServices

printf "Beginning PRPC installation...\n"
setPrpcMediaDir

ubu_installSunJDK
ubu_installOracle

configureOracle

#export prpc_connect="jdbc:oracle:thin:@${prpc_dbhost}:1521:${prpc_dbtsname}"
export prpc_connect="jdbc:oracle:thin:@${prpc_dbhost}:1521:${prpc_dbname}"
printf "\n\nYour connect string is: %s\n\n" ${prpc_connect}

tc_confdir=`find /etc -name "tomcat-users.xml" -type f -print | xargs -ifoo dirname foo`
tc_script=`find /etc/init.d -name "*omcat?" -type f -exec ls -1 {} \;`
tc_deploydir=`grep -i catalina_base\= ${tc_script} | cut -f 2 -d '=' | cut -f 1 -d '$' | xargs echo | cut -f 1 -d ' '`
tc_deploydir=`find $tc_deploydir -name "webapps" -type d -print`

printf "Top: Conf=%s Script=%s DeployDir=%s\n" $tc_confdir $tc_script $tc_deploydir

prepareTomcat 

# Load PRPC db with prdbutil application...
if [ -f ${prpc_mediadir}/archives/prdbutil.war ] ; then
    runRuleDBInstallServlet
else
    runRuleDBInstallScript
fi

printf "\n\nYou are now ready to login to PRPC on your VM Host\!\!\n\n\thttp://"`hostname`":${tc_svr_port_dist}/prweb/PRServlet\n\n"

### START PRESEED INCLUDE ###
###############################################################################
# Preseed file for automated PRPC packaged installation on Ubunutu
###############################################################################
# Locale sets language and country.
d-i debian-installer/locale string en_US
# Keyboard selection.
#d-i console-tools/archs select at
d-i console-keymaps-at/keymap select us
# Always install the virtual kernel, with no headers.
#d-i	base-installer/kernel/override-image	string linux-virtual
d-i	base-installer/kernel/override-image	string linux-generic-pae
d-i	base-installer/kernel/headers	boolean false
# Don't even install the standard task.
#tasksel	tasksel/first	multiselect server, lamp-server, openssh-server, tomcat-server, mail-server, samba-server
tasksel	tasksel/first	multiselect server, openssh-server, tomcat-server, mail-server, samba-server
# Only install basic language packs. Let tasksel ask about tasks.
d-i	pkgsel/language-pack-patterns	string
# No language support packages.
d-i	pkgsel/install-language-support	boolean false
# Use DHCP configuration to establish hostname and domain
d-i	netcfg/choose_interface	select auto
#d-i	netcfg/get_hostname	string unassigned-hostname
#d-i	netcfg/get_hostname	seen true
#d-i	netcfg/get_domain	string unassigned-domain
#d-i	netcfg/get_domain	seen true
#d-i	netcfg/dhcp_options	select Retry network autoconfiguration
# No automatic updates
d-i	pkgsel/update-policy	select none
# Turn off proxy configuration for mirrors - consider setting up a mirror 
# internallly for large deployments to manage updates
d-i	mirror/http/proxy	string 
# Only ask the UTC question if there are other operating systems installed.
d-i	clock-setup/utc-auto	boolean true
# You may set this to any valid setting for $TZ; see the contents of
# /usr/share/zoneinfo/ for valid values.
d-i	time/zone string US/Eastern
# No boot splash screen.
d-i	debian-installer/splash	boolean false
# Install the debconf oem-config frontend (if in OEM mode).
d-i	oem-config-udeb/frontend	string debconf
# Add the network and tasks oem-config steps by default.
oem-config	oem-config/steps	multiselect language, timezone, keyboard, user, network, tasks
# You can choose one of the three predefined partitioning recipes:
# - atomic: all files in one partition
# - home:   separate /home partition
# - multi:  separate /home, /usr, /var, and /tmp partitions
#d-i partman-auto/choose_recipe select atomic
# Define recipe to create a small /boot partition, suitable swap, and uses the
# rest of the space for the root partition - oracle needs at least 1024 M swap:
d-i	partman-auto/expert_recipe	string boot-root :: 24 48 512 ext3 $primary{ } $bootable{ } method{ format } format{ } use_filesystem{ } filesystem{ ext3 } mountpoint{ /boot } .  6696 8096 1000000000 ext3 method{ format } format{ } use_filesystem{ } filesystem{ ext3 } mountpoint{ / } .  1280 1794 200% linux-swap method{ swap } format{ } .
d-i	partman-auto/choose_recipe	select boot-root
# If the system has free space you can choose to only partition that space.
d-i	partman-auto/init_automatically_partition select biggest_free
# Only partition the first hard drive
#d-i	partman-auto/disk	string /dev/sda
# In addition, you'll need to specify the method to use.
# The presently available methods are: "regular", "lvm" and "crypto"
d-i	partman-auto/method string regular
# This makes partman automatically partition without confirmation, provided
# that you told it what to do using one of the methods above.
d-i	partman/confirm_write_new_label boolean true
d-i	partman/choose_partition select finish
d-i	partman/confirm boolean true
# To create a root account.
#d-i	passwd/root-login	boolean true
# To create a normal user account.
d-i	passwd/make-user	boolean true
d-i	passwd/user-fullname	string PRPC Administrator
d-i	passwd/username	string prpcadmin
#d-i	passwd/user-uid	string 1001
# Normal user's password, either in clear text
d-i	passwd/user-password	password <prpc_password>
d-i	passwd/user-password-again	password <prpc_password>
# The installer will warn about weak passwords. If you are sure you know
# what you're doing and want to override it, uncomment this.
d-i	user-setup/allow-password-weak	boolean true
# Set to true if you want to encrypt the first user's home directory.
d-i	user-setup/encrypt-home	boolean false
d-i	apt-setup/universe	boolean true
d-i	apt-setup/non-free	boolean true
d-i	apt-setup/partner	boolean true
d-i	apt-setup/restricted	boolean true
d-i	pkgsel/update-policy	select none
# Whether to upgrade packages after debootstrap.
# Allowed values: none, safe-upgrade, full-upgrade
d-i	pkgsel/upgrade	select full-upgrade
# Install service pkgs to support standalone PRPC svr config
d-i	pkgsel/include string bc libaio1 slapd ldap-utils postfix-doc postfix-ldap openssh-client portmap nfs-common wget curl finger sun-java6-jdk oracle-xe pegasystems-prpc
## ldap pam and nss preseeding
# Add the following packages if user account management on the server via LDAP
# is desired - append to previous pkgsel/include line...
#d-i	pkgsel/include	string libpam-ldap libnss-ldap nss-updatedb libnss-db
# libpam-ldap preseeding - enable ldap accounts in pam and nsswitch
#libnss-ldap	libnss-ldap/bindpw	password <ldapadmpasswd>
#libpam-ldap	libpam-ldap/rootbindpw	password <ldapadmpasswd>
#libnss-ldap	libnss-ldap/rootbindpw	password <ldapadmpasswd>
#libpam-ldap	libpam-ldap/bindpw	password <ldapadmpasswd>
#libnss-ldap	libnss-ldap/dblogin	boolean	false
#libnss-ldap	libnss-ldap/override	boolean	true
#libnss-ldap	shared/ldapns/base-dn	string dc=<sys_domain1>,dc=<sys_domain2>
#libpam-ldap	shared/ldapns/base-dn	string dc=<sys_domain1>,dc=<sys_domain2>
#libnss-ldap	shared/ldapns/ldap-server	string ldaps://localhost
#libpam-ldap	shared/ldapns/ldap-server	string ldaps://localhost
#libpam-ldap	libpam-ldap/pam_password	select crypt
#libpam-ldap	libpam-ldap/binddn	string cn=ldapadm,dc=<sys_domain1>,dc=<sys_domain2>
#libnss-ldap	libnss-ldap/confperm	boolean false
#libpam-ldap	libpam-ldap/rootbinddn	string cn=ldapadm,dc=<sys_domain1>,dc=<sys_domain2>
#libpam-ldap	libpam-ldap/dbrootlogin	boolean true
#libnss-ldap	libnss-ldap/rootbinddn	string cn=ldapadm,dc=<sys_domain1>,dc=<sys_domain2>
#libpam-ldap	libpam-ldap/override	boolean true
#libnss-ldap	shared/ldapns/ldap_version	select 3
#libpam-ldap	shared/ldapns/ldap_version	select 3
#libpam-ldap	libpam-ldap/dblogin	boolean false
#libnss-ldap	libnss-ldap/binddn	string cn=ldapadm,dc=<sys_domain1>,dc=<sys_domain2>
#libnss-ldap	libnss-ldap/nsswitch	note 
#libnss-ldap	libnss-ldap/dbrootlogin	boolean true
## sun-java6-jdk preseeding - accept the license 
sun-java6-jdk	shared/accepted-sun-dlj-v1-1	boolean true
## postfix preseeding
# General type of configuration? Default:Internet Site
# Choices: No configuration, Internet Site, Internet with smarthost,
#   Satellite system, Local only
postfix	postfix/main_mailer_type	select Internet Site
postfix postfix/destinations string <sys_hostname>.<sys_domain>, localhost
postfix postfix/mailname string <sys_hostname>.<sys_domain>
# Where should mail for root go, Default:if not set, will spool locally
postfix	postfix/root_address	string prpcadmin@<sys_domain>
# SMTP relay host? (blank for none) Default:(none)
#postfix	postfix/relayhost	string smtp.<sys_domain>
# Force synchronous updates on mail queue?  Default:false
#postfix postfix/chattr  boolean true
# Local networks? Default:"127.0.0.0/8" 
# blank uses the postfix default (which is based on the connected subnets)
#postfix postfix/mynetworks      string  
# Use procmail for local delivery? Defaults to true if /usr/bin/procmail exists
#postfix postfix/procmail        boolean false
# Mailbox size limit Default:0 (unlimited), upstream default is 51200000
postfix postfix/mailbox_limit   string  51200000
# Local address extension character? Default:+
#postfix postfix/recipient_delim string  -
# Internet protocols to use? Default is based on checking if
#  /proc/sys/net/ipv{4,6} exist
# Choices: all, ipv6, ipv4
postfix postfix/protocols       select  ipv4
# mysql-server configuration - only needed if enabling lamp-server task
#mysql-server-5.0 mysql-server/root_password string
#mysql-server-5.0 mysql-server/root_password seen true
#mysql-server-5.0 mysql-server/root_password_again string
#mysql-server-5.0 mysql-server/root_password_again seen true
#mysql-server-5.0 mysql-server/root_password password <prpc_password>
#mysql-server-5.0 mysql-server/root_password_again password <prpc_password>
# This is fairly safe to set, it makes grub install automatically to the MBR
# if no other operating system is detected on the machine.
d-i	grub-installer/only_debian	boolean true
d-i	preseed/late_command	string cp /cdrom/prpc-firstrun /target/etc/init.d/prpc-firstrun; chmod 755 /target/etc/init.d/prpc-firstrun